
/*
 * $Id$
 *
 * (C) by abook developers
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
#	include "../config.h"
#endif
#include "../abook.h"
#include "../db_access.h"
#include "../misc.h"
#include "../xmalloc.h"
#include <assert.h>

#include "filter_interface.h"
#include "filter.h"

static int
abook_parse_database(FILE *in)
{
        char *line = NULL;
	char *tmp;
	int sec=0, field;
	list_item item;

	item = item_create();

	for(;;) {
		line = getaline(in);
		if(feof(in)) {
			if(item[field_id(NAME)] && sec) {
				add_item2database(item);
			} else {
				item_empty(item);
			}
			break;
		}

		if(!*line || *line == '\n' || *line == '#') {
			goto next;
		} else if(*line == '[') {
			if(item[field_id(NAME)] && sec ) {
				add_item2database(item);
			} else {
				item_empty(item);
			}
			sec = 1;
			item_blank(item);
			if(!(tmp = strchr(line, ']')))
				sec = 0; /*incorrect section lines are skipped*/
		} else if((tmp = strchr(line, '=') ) && sec) {
			*tmp++ = '\0';
			if((field = field_index(line)) != -1) {
				item[field] = xstrdup(tmp);
				goto next;
			} else if(!strcasecmp(opt_get_str(STR_PRESERVE_FIELDS),
						"all")) {
				int last = declare_unknown_field(line);
				assert(last != -1);
				item_resize(&item);
				item[last] = xstrdup(tmp);
				goto next;
			}
		}
next:
		xfree(line);
	}

	xfree(line);
	item_free(&item);
	return 0;
}

static int
abook_write_database(FILE *out, struct db_enumerator e)
{
	int i = 0;
	struct field_enumerator fe = init_field_enumerator(ENUM_ALL);
	char *str;

	fprintf(out,
		"# abook addressbook file\n\n"
		"[format]\n"
		"program=" PACKAGE "\n"
		"version=" VERSION "\n"
		"\n\n"
	);

	db_enumerate_items(e) {
		fprintf(out, "[%d]\n", i);

		enumerate_fields(fe) {
			str = db_fget_byid(e.item, fe.index);
			if(str && *str)
				fprintf(out,"%s=%s\n",field_key(fe.index),str);
		}

		fputc('\n', out);
		i++;
	}

	return 0;
}

/*
 * Using filter helper
 */
#define FILTER_NAME abook
#define FILTER_DESCRIPTION N_("abook native format")
#define FILTER_IMPORT_FUNCTION abook_parse_database
#define FILTER_EXPORT_FUNCTION abook_write_database
#include "filter_helper.h"
